package org.apache.ojb.broker.metadata;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache ObjectRelationalBridge" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache ObjectRelationalBridge", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/**
 * This interface contains integer constants representing the
 * elements of a DescriptorRepository.
 * This constants are used in marshalling and unmarshalling a
 * DescriptorRepository to identify all its constituent elements.
 * @author      Thomas Mahler
 */
public interface RepositoryElements
{
    public static final int MappingRepository = 0;
    public static final int JdbcConnectionDescriptor = 1;
    public static final int Dbms_name = 2;
    public static final int SORT = 3;
    //public static final int Dbms_port = 4;
    public static final int Schema_name = 5;
    public static final int Driver_name = 6;
    public static final int Url_protocol = 7;
    public static final int Url_subprotocol = 8;
    public static final int Url_dbalias = 9;
    public static final int User_name = 10;
    public static final int User_passwd = 11;
    public static final int ClassDescriptor = 12;
    public static final int Class_name = 13;
    public static final int Class_proxy = 35;
    public static final int Class_extent = 33;
    public static final int Table_name = 14;
    public static final int Orderby = 36;
    public static final int FieldConversion = 30;
    public static final int RowReader = 32;
    public static final int FIELDDESCRIPTOR = 15;
    public static final int Field_name = 16;
    public static final int Column_name = 17;
    public static final int Jdbc_type = 18;
    public static final int PrimaryKey = 19;
    public static final int Autoincrement = 31;
    public static final int ReferenceDescriptor = 20;
    //public static final int Rdfield_name = 21;
    public static final int Referenced_class = 22;
    public static final int Fk_descriptor_id = 23;
    public static final int Auto_retrieve = 24;
    public static final int Auto_update = 25;
    public static final int Auto_delete = 26;
    public static final int CollectionDescriptor = 27;
    //public static final int Cdfield_name = 28;
    public static final int Items_class = 29;
    public static final int Inverse_fk = 38;
    public static final int Collection_class = 37;
    public static final int Indirection_table = 39;
    public static final int Fk_pointing_to_items_class = 40;
    public static final int Fk_pointing_to_this_class = 41;
    //public static final int ProxyCollection = 42;
    //public static final int RefreshCollection = 43;
    public static final int Datasource_name = 44;
    public static final int Jdbc_level = 45;
    public static final int Locking = 46;
    public static final int RefreshReference = 47;
    public static final int ProxyReference = 48;
    public static final int ISOLATION_LEVEL = 34;
    public static final int FOREIGN_KEY = 49;
    public static final int NULLABLE = 50;
    public static final int INDEXED = 51;
    public static final int LENGTH = 52;
    public static final int PRECISION = 53;
    public static final int SCALE = 54;
    

    public static final int conMaxActive = 55;
    public static final int conMaxIdle = 56;
    public static final int conMaxWait = 57;
    public static final int conMinEvictableIdleTimeMillis = 58;
    public static final int conNumTestsPerEvictionRun = 59;
    public static final int conTestOnBorrow = 60;
    public static final int conTestOnReturn = 61;
    public static final int conTestWhileIdle = 62;
    public static final int conTimeBetweenEvictionRunsMillis = 63;
    public static final int conWhenExhaustedAction = 64;
    public static final int connection_pool = 65;
    public static final int connectionFactory = 66;


    
    public static final int RepositoryFragment = 67; // MF

    // maintain a last id to keep track where we are
    public static final int _LAST = 68;

}
