package org.apache.ojb.broker.metadata;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache ObjectRelationalBridge" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache ObjectRelationalBridge", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import org.apache.ojb.broker.util.DoubleHashtable;

/**
 * this class maintains a table mapping the xml-tags used in the
 * repository.dtd to their corresponding ids used within OJB.
 * This table is used in <br>
 * 1. the RepositoryXmlHandler to identify tags on parsing the
 * repository.xml in a large switch statement.
 * 2. in the RepositoryPersistor to get the proper tag for a
 * given XmlCapable id during assembling the repository.xml
 * for output.<br>
 * <b>Important note: This class is the only place where XML tags from the
 * repository.dtd are maintained.
 * All usages of these tags within OJB must use this table to ease
 * changes of the DTD.</b>
 * @author      Thomas Mahler
 */
public class RepositoryTags implements RepositoryElements
{
    /**
     * the two-way hashtable holding all entries.
     */
    private DoubleHashtable table;

    /**
     * the singleton instance of this class.
     */
    private static RepositoryTags instance = new RepositoryTags();

    /**
     * private Constructor, please use getInstance() to obtain
     * the singleton instance of this class.
     */
    private RepositoryTags()
    {
        // construct the mapping table
        table = new DoubleHashtable();
        table.put("descriptor-repository", new Integer(MappingRepository));
        table.put("repository-fragment", new Integer(RepositoryFragment));
        table.put("jdbc-connection-descriptor", new Integer(JdbcConnectionDescriptor));
        table.put("platform", new Integer(Dbms_name));
        table.put("schema", new Integer(Schema_name));
        table.put("driver", new Integer(Driver_name));
        table.put("protocol", new Integer(Url_protocol));
        table.put("subprotocol", new Integer(Url_subprotocol));
        table.put("dbalias", new Integer(Url_dbalias));
        table.put("username", new Integer(User_name));
        table.put("password", new Integer(User_passwd));
        table.put("class-descriptor", new Integer(ClassDescriptor));
        table.put("class", new Integer(Class_name));
        table.put("proxy", new Integer(Class_proxy));
        table.put("extent-class", new Integer(Class_extent));
        table.put("table", new Integer(Table_name));
        table.put("orderby", new Integer(Orderby));
        table.put("conversion", new Integer(FieldConversion));
        table.put("row-reader", new Integer(RowReader));
        table.put("field-descriptor", new Integer(FIELDDESCRIPTOR));
        table.put("name", new Integer(Field_name));
        table.put("column", new Integer(Column_name));
        table.put("jdbc-type", new Integer(Jdbc_type));
        table.put("primarykey", new Integer(PrimaryKey));
        table.put("autoincrement", new Integer(Autoincrement));
        table.put("nullable", new Integer(NULLABLE));
        table.put("indexed", new Integer(INDEXED));
        table.put("length", new Integer(LENGTH));
        table.put("precision", new Integer(PRECISION));
        table.put("scale", new Integer(SCALE));

        table.put("reference-descriptor", new Integer(ReferenceDescriptor));
        //table.put("rdfield.name", new Integer(Rdfield_name));
        table.put("class-ref", new Integer(Referenced_class));
        table.put("foreignkey", new Integer(FOREIGN_KEY));
        table.put("field-id-ref", new Integer(Fk_descriptor_id));
        table.put("auto-retrieve", new Integer(Auto_retrieve));
        table.put("auto-update", new Integer(Auto_update));
        table.put("auto-delete", new Integer(Auto_delete));
        table.put("collection-descriptor", new Integer(CollectionDescriptor));
        //table.put("cdfield.name", new Integer(Cdfield_name));
        table.put("element-class-ref", new Integer(Items_class));
        table.put("inverse-foreignkey", new Integer(Inverse_fk));
        table.put("collection-class", new Integer(Collection_class));
        table.put("indirection-table", new Integer(Indirection_table));
        table.put("fk-pointing-to-element-class", new Integer(Fk_pointing_to_items_class));
        table.put("fk-pointing-to-this-class", new Integer(Fk_pointing_to_this_class));
        //table.put("proxyCollection", new Integer(ProxyCollection));
        //table.put("refreshCollection", new Integer(RefreshCollection));
        table.put("jndi-datasource-name", new Integer(Datasource_name));
        table.put("jdbc-level", new Integer(Jdbc_level));
        table.put("locking", new Integer(Locking));
        table.put("refresh", new Integer(RefreshReference));
        table.put("proxy", new Integer(ProxyReference));
        table.put("isolation-level", new Integer(ISOLATION_LEVEL));
        table.put("sort", new Integer(SORT));

        table.put("maxActive", new Integer(conMaxActive));
        table.put("maxIdle", new Integer(conMaxIdle));
        table.put("maxWait", new Integer(conMaxWait));
        table.put("minEvictableIdleTimeMillis", new Integer(conMinEvictableIdleTimeMillis));
        table.put("numTestsPerEvictionRun", new Integer(conNumTestsPerEvictionRun));
        table.put("testOnBorrow", new Integer(conTestOnBorrow));
        table.put("testOnReturn", new Integer(conTestOnReturn));
        table.put("testWhileIdle", new Integer(conTestWhileIdle));
        table.put("timeBetweenEvictionRunsMillis", new Integer(conTimeBetweenEvictionRunsMillis));
        table.put("whenExhaustedAction", new Integer(conWhenExhaustedAction));
        table.put("connectionFactory", new Integer(connectionFactory));
        table.put("connection-pool", new Integer(connection_pool));

        // add new tags here !
    }

    /**
     * returns the singleton instance.
     */
    public static RepositoryTags getInstance()
    {
        return instance;
    }

	/**
	 * returns the xml-tag literal associated with the repository element with
	 * id <code>elementId</code>.
	 * @return the resulting tag
	 */
    public String getTagById(int elementId)
    {
        return (String) table.getKeyByValue(new Integer(elementId));
    }

	/**
	 * returns the opening xml-tag associated with the repository element with
	 * id <code>elementId</code>.
	 * @return the resulting tag
	 */
    public String getOpeningTagById(int elementId)
    {
        return "<" + table.getKeyByValue(new Integer(elementId)) + ">";
    }

	/**
	 * returns the opening xml-tag associated with the repository element with
	 * id <code>elementId</code>.
	 * @return the resulting tag
	 */
    public String getOpeningTagById(int elementId, String attributes)
    {
        return "<" + table.getKeyByValue(new Integer(elementId)) + " " + attributes + ">";
    }

	/**
	 * returns the closing xml-tag associated with the repository element with
	 * id <code>elementId</code>.
	 * @return the resulting tag
	 */
    public String getClosingTagById(int elementId)
    {
        return "</" + table.getKeyByValue(new Integer(elementId)) + ">";
    }

    /**
     * returns the repository element id associated with the xml-tag
     * literal <code>tag</code>.
     * @return the resulting repository element id.
     * @throws NullPointerException if no value was found for <b>tag</b>
     */
    public int getIdByTag(String tag)
    {
        return ((Integer) table.getValueByKey(tag)).intValue();
    }

	/**
	 * returns the opening xml-tag associated with the repository element with
	 * id <code>elementId</code>.
	 * @return the resulting tag
	 */
    public String getCompleteTagById(int elementId, String characters)
    {
        String result = getOpeningTagById(elementId);
        result += characters;
        result += getClosingTagById(elementId);
        return result;
    }


}
