package org.apache.ojb.broker.util.configuration.impl;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache ObjectRelationalBridge" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache ObjectRelationalBridge", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import org.apache.ojb.broker.ManageableCollection;
import org.apache.ojb.broker.PersistenceBroker;
import org.apache.ojb.broker.ta.PBPoolConfiguration;
import org.apache.ojb.broker.platforms.PlatformConfiguration;
import org.apache.ojb.broker.accesslayer.ConnectionFactory;
import org.apache.ojb.broker.accesslayer.ConnectionFactoryDefaultImpl;
import org.apache.ojb.broker.accesslayer.ConnectionFactoryConfiguration;
import org.apache.ojb.broker.cache.ObjectCache;
import org.apache.ojb.broker.cache.ObjectCacheDefaultImpl;
import org.apache.ojb.broker.metadata.MetadataConfiguration;
import org.apache.ojb.broker.metadata.PersistentField;
import org.apache.ojb.broker.metadata.PersistentFieldDefaultImpl;
import org.apache.ojb.broker.server.BrokerPool;
import org.apache.ojb.broker.server.PersistenceBrokerServer;
import org.apache.ojb.broker.singlevm.PersistenceBrokerConfiguration;
import org.apache.ojb.broker.singlevm.PersistenceBrokerImpl;
import org.apache.ojb.broker.util.logging.Logger;
import org.apache.ojb.broker.util.logging.LoggingConfiguration;
import org.apache.ojb.broker.util.logging.PoorMansLoggerImpl;
import org.apache.ojb.broker.util.sequence.SequenceConfiguration;
import org.apache.ojb.broker.util.sequence.SequenceManager;
import org.apache.ojb.broker.util.sequence.SequenceManagerDefaultImpl;
import org.apache.ojb.odmg.OdmgConfiguration;
import org.apache.ojb.odmg.collections.DListImpl;
import org.apache.ojb.odmg.locking.AbstractLockStrategy;
import org.apache.ojb.odmg.locking.LockManager;
import org.apache.ojb.odmg.locking.LockManagerDefaultImpl;
import org.apache.ojb.odmg.locking.LockMap;
import org.apache.ojb.odmg.locking.LockingConfiguration;
import org.apache.ojb.odmg.locking.PersistentLockMapImpl;

/**
 * This class contains the runtime configuration of the OJB
 * system. This Configuration is read in only once at application startup.
 * Changes to the OJB.properties file during execution are <b>not</b>
 * reflected back into the application!
 *
 * @author Thomas Mahler
 * @version $Id: OjbConfiguration.java,v 1.5 2002/07/30 17:23:55 arminw Exp $
 */
public class OjbConfiguration
    extends ConfigurationAbstractImpl
    implements LoggingConfiguration, OdmgConfiguration, LockingConfiguration,
               PersistenceBrokerConfiguration, SequenceConfiguration, PlatformConfiguration,
               PBPoolConfiguration, ConnectionFactoryConfiguration, MetadataConfiguration
{
    /** use Server or local broker*/
    private boolean serverUsed;

    /** the repository file keeping the O/R Metadata*/
    private String repositoryFilename;

    private String[] servers;

    private Class sequenceManagerClass;

    private Class objectCacheClass;

    private Class connectionFactoryClass;

    private Class lockManagerClass;

    private Class lockMapClass;

    private boolean lockAssociationAsWrites;

    private int sequenceManagerGrabSize;

    private Class oqlCollectionClass;

    private Class loggerClass;

    private Class persistentFieldClass;

    private String loggerConfigFile;

    private Class persistenceBrokerClass;

	private boolean ignoreAutocommitExceptions;

	private boolean useAutoCommit;

	private int prefetchInLimit;

    // PB pooling configuration
    private int maxActive;
    private int maxIdle;
    private long maxWait;
    private long timeBetweenEvictionRunsMillis;
    private long minEvictableIdleTimeMillis;

    // Connection pooling configuration
    private int con_maxActive;


	// dynamic repository loading
	private boolean autocreateDynamicTables;
	
    public OjbConfiguration()
    {
    }


    public String getLoggerConfigFile()
    {
        return loggerConfigFile;
    }


    //*************************************************************
    //Connection pooling configuration methods
    public int getConMaxActive()
    {
        return con_maxActive;
    }

    //*************************************************************
    //PBPoolConfiguration methods
    public int getMaxActive()
    {
        return maxActive;
    }

    public int getMaxIdle()
    {
        return maxIdle;
    }

    public long getMaxWaitMillis()
    {
        return maxWait;
    }

    public long getTimeBetweenEvictionRunsMilli()
    {
        return timeBetweenEvictionRunsMillis;
    }

    public long getMinEvictableIdleTimeMillis()
    {
        return minEvictableIdleTimeMillis;
    }
    //*************************************************************


    public String getLogLevel(String loggerName)
    {
        return this.getString(loggerName + ".LogLevel", "INFO");
    }

    public boolean lockAssociationAsWrites()
    {
        return lockAssociationAsWrites;
    }

    public boolean isRunningInServerMode()
    {
        return serverUsed;
    }

    public String getRepositoryFilename()
    {
        return repositoryFilename;
    }

    public String[] getServers()
    {
        return servers;
    }

    public Class getSequenceManagerClass()
    {
        return sequenceManagerClass;
    }

    public Class getObjectCacheClass()
    {
        return objectCacheClass;
    }

    public Class getConnectionFactoryClass()
    {
        return connectionFactoryClass;
    }

    public Class getLockManagerClass()
    {
        return lockManagerClass;
    }

    public int getSequenceManagerGrabSize()
    {
        return sequenceManagerGrabSize;
    }

    public Class getOqlCollectionClass()
    {
        return oqlCollectionClass;
    }

    public Class getLockMapClass()
    {
        return lockMapClass;
    }

    public Class getLoggerClass()
    {
        return loggerClass;
    }

    public Class getPersistentFieldClass()
    {
        return persistentFieldClass;
    }

    public Class getPersistenceBrokerClass()
    {
        return persistenceBrokerClass;
    }

	public boolean ignoreAutocommitExceptions()
	{
		return ignoreAutocommitExceptions;
	}

	public boolean useAutoCommit()
	{
		return useAutoCommit;
	}
    /**
     * loads the configuration
     */
    protected void load()
    {
        // properties file may be set as a System property.
        // if no property is set take default name.
        String fn = System.getProperty("OJB.properties", "OJB.properties");
        setFilename(fn);
        super.load();

        //Usage of local or remote Broker
        serverUsed = getBoolean("useServer", false);
        repositoryFilename = getString("repositoryFile", "repository.xml");
        servers = getStrings("BrokerServers", "localhost:2001");

        // load SeqManager class
        sequenceManagerClass =
            getClass("SequenceManagerClass", SequenceManagerDefaultImpl.class, SequenceManager.class);

        // load SeqManager Grab size value
        sequenceManagerGrabSize = getInteger("SequenceManagerGrabSize", 10);

        // load ObjectCache Class
        objectCacheClass = getClass("ObjectCacheClass", ObjectCacheDefaultImpl.class, ObjectCache.class);

        // load ConnectionFactory Class
        connectionFactoryClass =
            getClass("ConnectionFactoryClass", ConnectionFactoryDefaultImpl.class, ConnectionFactory.class);

        // load LockManager Class
        lockManagerClass = getClass("LockManagerClass", LockManagerDefaultImpl.class, LockManager.class);

        // load LockMap Class
        lockMapClass = getClass("LockMapClass", PersistentLockMapImpl.class, LockMap.class);

        // load Logger Class
        loggerClass = getClass("LoggerClass", PoorMansLoggerImpl.class, Logger.class);
        loggerConfigFile = getString("LoggerConfigFile", "");

        // load PersistentField Class
        persistentFieldClass =
            getClass("PersistentFieldClass", PersistentFieldDefaultImpl.class, PersistentField.class);

        // load PersistenceBroker Class
        persistenceBrokerClass =
            getClass("PersistenceBrokerClass", PersistenceBrokerImpl.class, PersistenceBroker.class);

        // load Locking Timeout
        AbstractLockStrategy.DEFAULT_LOCK_TIMEOUT = getInteger("LockTimeout", 60000);

        // load number of worker threads for the PersistenceBrokerServer
        PersistenceBrokerServer.THREADCOUNT = getInteger("ServerThreadCount", 10);

        // load number of Brokers in the BrokerPool of the PersistenceBrokerServer
        BrokerPool.BROKERCOUNT = getInteger("BrokerPoolSize", 10);

        // load configuration for LockAssociations parameter:
        lockAssociationAsWrites =
            (getString("LockAssociations", "WRITE").equalsIgnoreCase("WRITE") ? true : false);

        // load OQL Collection Class
        oqlCollectionClass = getClass("OqlCollectionClass", DListImpl.class, ManageableCollection.class);

		// load the platform configuration for autocommit exceptions
		ignoreAutocommitExceptions = super.getBoolean("ignoreAutoCommitExceptions",false);

		useAutoCommit = getBoolean("useAutoCommit", true);

        // set the limit for IN-sql used in prefetch, -1 for no limits
		prefetchInLimit = getInteger("PrefetchInLimit", -1);

        //load configuration for PB pool
        maxActive = getInteger("maxActive", 100);
        maxIdle = getInteger("maxIdle", 50);
        maxWait = (long) getInteger("maxWait", 2000);
        timeBetweenEvictionRunsMillis = (long) getInteger("timeBetweenEvictionRunsMillis", -1);
        minEvictableIdleTimeMillis = (long) getInteger("minEvictableIdleTimeMillis", 1000000);

        //load configuration for connection pools
        con_maxActive = getInteger("maxConnectionsInPool", 10);
        
        // load configuration for metadata
        autocreateDynamicTables = getBoolean("autocreateDynamicTables", false);
    }
    
	/**
	 * Returns the prefetchInLimit.
	 * @return int
	 */
	public int getPrefetchInLimit()
	{
		return prefetchInLimit;
	}

	/**
	 * @return true if OJB should attempt to create DB tables for dynamically loaded class descriptors
	 */
	public boolean allowAutocreateDynamicTables() {
		return autocreateDynamicTables;
	}
}