package org.apache.ojb.broker.metadata;

import junit.framework.TestCase;
import org.apache.ojb.broker.PBKey;
import org.apache.ojb.broker.PersistenceBroker;
import org.apache.ojb.broker.PersistenceBrokerException;
import org.apache.ojb.broker.PersistenceBrokerFactory;
import org.apache.ojb.broker.query.Criteria;
import org.apache.ojb.broker.query.EqualToCriteria;
import org.apache.ojb.broker.query.Query;
import org.apache.ojb.broker.query.QueryFactory;
import org.apache.ojb.broker.util.logging.*;
import org.apache.ojb.broker.util.logging.LoggerFactory;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.TreeSet;

/**
 * A test case for dynamic repository loading.
 *
 * @author: Martin Fuzzey
 */
public class DynamicRepositoryTest extends TestCase
{
    PersistenceBroker broker;

    int testId = 12345;
	private PBKey pbKey = new PBKey("repository.xml");

    private static Class CLASS = DynamicRepositoryTest.class;

    /**
     * BrokerTests constructor comment.
     * @param name java.lang.String
     */
    public DynamicRepositoryTest(String name)
    
    {
        super(name);
    }


    /**
     * Insert the method's description here.
     * Creation date: (23.12.2000 18:30:38)
     * @param args java.lang.String[]
     */
    public static void main(String[] args)
    {
        String[] arr = { CLASS.getName()};
        junit.textui.TestRunner.main(arr);
    }

	
    /**
     * Insert the method's description here.
     * Creation date: (06.12.2000 21:58:53)
     */
    public void setUp()
    {
        try
        {
            broker = PersistenceBrokerFactory.createPersistenceBroker(pbKey);
        }
        catch (Throwable t)
        {
            LoggerFactory.getBootLogger().error("error getting broker", t);
        }
    }


    /**
     * Insert the method's description here.
     * Creation date: (06.12.2000 21:59:14)
     */
    public void tearDown()
    {
        try
        {
            broker.close();
        }
        catch (Throwable t)
        {
        }
    }
    
    private TreeSet expectedObjs = new TreeSet(); // for sorted
    
    private void store(Object obj) {
        expectedObjs.add(obj);
        broker.store(obj);
    }
    
    private Collection readAll() {
    	Criteria crit = null;
        Query q = QueryFactory.newQuery(DynamicObject.class, crit);
        
        return broker.getCollectionByQuery(q);
    }

    private void checkExpected() {
    	Collection got = readAll();
    	if (got != null) got = new TreeSet(got); // sort
        assertEquals(
               "stored objects should be as expected",
                expectedObjs,
                got);
    }

    /**
     * Insert the method's description here.
     * Creation date: (06.12.2000 21:51:22)
     */
    public void testDynamic()
    {
        try
        {
            for (int i=1; i<3; i++) {
                DynamicNumber num = new DynamicNumber(i);
                store(num);
                DynamicString str = new DynamicString("val" + i);
                store(str);
                DynamicAutoCreateNumber num2 = new DynamicAutoCreateNumber(i);
                store(num2);
                DynamicAutoCreateString str2 = new DynamicAutoCreateString("val" + i);
                store(str);
            }
            
            // check ok
            checkExpected();
            
            // check ok after resetting PB
            broker.getDescriptorRepository().reset();
            checkExpected();
        }
        catch (Throwable t)
        {
            LoggerFactory.getDefaultLogger().error(t);
            fail(t.getMessage());
        }
    }

    /**
     * Test MN relation for dynamically loaded descriptors
     */
    public void testDynamicMN()
    {
        try
        {
        	DynamicContainer cont = new DynamicContainer("cont1");
        	cont.add(new DynamicString("item1"));
        	cont.add(new DynamicString("item2"));
            store(cont);

            // check ok
            checkExpectedContainer("cont1", cont);
            
            // check ok after resetting PB
            broker.getDescriptorRepository().reset();
            checkExpectedContainer("cont1", cont);
        }
        catch (Throwable t)
        {
            LoggerFactory.getDefaultLogger().error(t);
            fail(t.getMessage());
        }
    }

	private void checkExpectedContainer(String name, DynamicContainer exp) {
		DynamicContainer got = readContainer(name);
		assertEquals("Container contents", exp.toString(), got.toString());
	}
	
    private DynamicContainer readContainer(String name) {
    	Criteria crit = new Criteria();
    	crit.addEqualTo("name", name);
        Query q = QueryFactory.newQuery(DynamicContainer.class, crit);
        
        Collection containers = broker.getCollectionByQuery(q);
        assertEquals("only expected single container",1,  containers.size());
        return (DynamicContainer)containers.iterator().next();
    }

}
